/* miscFunctions.ads
 * A plethora of miscellaneous, useful functions.
 * Most were written by Marcin S for Dan Cook's Antidisassemblage language.
 * Most of the RandByte() code was written by Joe Wingbermuehle
 * Testing and such things as defining key and group macros are not finished yet.
 * 
 *
 * List of contents:
 *
 *  - .AsmInit() - macro
 *  - .ScrollOff() - macro
 *  - .ScrollUp() - macro
 *  - .SetContrast() - macro
 *
 *  - Group and Key values defined for KeyInput()
 *
 *  - byte KeyInput(byte keyGroup)
 *  - void Delay(byte delayLength)
 *  - byte RandByte(byte lowerBound, byte upperBound)
 */


byte contrast @ "contrast";

//=========
// Macro that does useful things normally done in asm progs

#define "AsmInit ( )" as @" DI\n b_call(_RunIndicOff)\n RES DonePrgm, (IY + DoneFlags)"

//=========
// Macro that turns scrolling off

#define "ScrollOff ( )" as @" RES AppAutoScroll, (IY + AppFlags)"

//=========
// Macro that scrolls up once (turns scrolling on, if you don't want this just
//  call .ScrollOff())
// CurRow and CurCol set to 7

#define "ScrollUp ( )" as @" SET AppAutoScroll, (IY + AppFlags)
 \n LD HL, $0707\n LD (CurRow), HL\n b_call(_NewLine)"

//=========
// Macro that sets contrast, acceptable values are 0 - 39

#define "SetContrast (", setContrast, ")" as "contrast = ", setContrast, ";", 
 @" LD A, (contrast)\n ADD A, $D8\n OUT ($10), A"



//=========
// key values and group values for KeyInput() are defined here
// When using in a program, make sure to precede these with a ".", as these are macros.
// Not all key and group values are defined yet, but these are the main ones.

#define "GROUP_DOWN" as "254"
#define "GROUP_LEFT" as "254"
#define "GROUP_RIGHT" as "254"
#define "GROUP_UP" as "254"
#define "GROUP_ARROWS" as "254"

#define "KEY_DOWN" as "254"
#define "KEY_LEFT" as "253"
#define "KEY_RIGHT" as "251"
#define "KEY_UP" as "247"

#define "GROUP_2ND" as "191"
#define "GROUP_DEL" as "191"
#define "GROUP_GRAPH" as "191"

#define "KEY_2ND" as "223"
#define "KEY_DEL" as "127"

#define "GROUP_ALPHA" as "223"
#define "GROUP_MATH" as "223"

#define "KEY_ALPHA" as "127"
#define "KEY_MATH" as "191"

#define "GROUP_CLEAR" as "253"
#define "GROUP_ENTER" as "253"

#define "KEY_CLEAR" as "191"
#define "KEY_ENTER" as "254"



//=========
// Function for key input, much faster than GetKey
// The parameter is the group of the desired key to be tested for,
//  e.g. if you want to check if DOWN is pressed, set the parameter to .GROUP_DOWN
// The function returns the value of the key that was pressed in that group;
//  to check if DOWN was pressed, do something like 
//  byte key = KeyInput(.GROUP_ARROWS);
//  if ( key == .KEY_DOWN )
//  ... code here

byte KeyInput(byte keyGroup)
{
  #asm " LD A, ", keyGroup, "\n OUT (1), A\n NOP\n NOP"
  #asm " IN A, (1)\n LD ", keyGroup, ", A"
  this = keyGroup;
}

//=========
// Delay function - accepts values from 0 - 255
// Brief delay to slow down programs/games. For longer delays put inside a loop
// (I've discovered that memory-wise this is not the most efficient delay, but oh well)

void Delay(byte delayLength)
{
 #asm " LD A, ", delayLength, "\n LD B, A"
 #asm "Label_Delay_Loop:\n LD C, B\n LD B, 255"
 #asm "Label_Delay_Loop_2:\n NOP\n DJNZ Label_Delay_Loop_2"
 #asm " LD B, C\n DJNZ Label_Delay_Loop"
}

//=========
// Random Number function for bytes
// This is the routine that Ion uses for random numbers, slightly modified.
// Will return a random byte between lowerBound and upperBound, inclusive.

byte RandByte(byte lowerBound, byte upperBound)
{
 word randData;
 upperBound = upperBound - lowerBound;
 #asm " LD A, ", upperBound, "\n INC A\n LD B, A"
 #asm "random:\n PUSH HL\n PUSH DE\n LD HL, ", randData, "\n LD A, R\n LD D, A"
 #asm " LD E, (HL)\n ADD HL, DE\n ADD A, L\n XOR H\n LD ", randData, ", HL"
 #asm " SBC HL, HL\n LD E, A\n LD D, H\nrandomLoop:\n ADD HL, DE\n DJNZ randomLoop"
 #asm " LD A, H\n POP DE\n POP HL"
 #asm " LD ", upperBound, ", A"
 this = upperBound + lowerBound;
}

#eof
